unit FObj;

interface

uses
  SysUtils, Classes, Graphics, Controls, WinProcs, WinTypes;

const

  acCopy     =  1;
  acDelete   =  2;
  acMove     =  4;
  acMkDir    =  8;
  acRmDir    = 16;
  acSetAttr  = 32;
  acUnselect = 64;

type

  TSortType = (stNone,stName,stExt,stSize,stTime);
  TAttrStr=string[4];
  TFileRec = class(TObject)
   Attr: Byte;
   Time: Longint;
   Size: Longint;
   Name: string[12];
   Ext : string[3];
   Selected : boolean;
   Action : byte;
   procedure SetValues(S:TSearchRec);
   function GetString(Sort:TSortType):string;
   function GetName:string;
   function GetNameStr:string;
   function GetExtStr:string;
   function GetSizeStr:string;
   function GetTimeStr:string;
   function GetSortSize:string;
   function GetSortTime:string;
   function GetAtribStr:TAttrStr;
   procedure SplitNameExt;
  end;

implementation

function TFileRec.GetExtStr:string;
 begin
  if (Attr and faDirectory)>0 then GetExtStr:=''
                              else GetExtStr:=LowerCase(Ext);
 end;

function TFileRec.GetNameStr:string;
 begin
  if (Attr and faDirectory)>0 then
   if Ext='' then GetNameStr:='['+LowerCase(Name)+']'
             else GetNameStr:='['+LowerCase(Name+'.'+Ext)+']'
  else GetNameStr:=LowerCase(Name);
 end;

function TFileRec.GetTimeStr:string;
 var lStr:string;
     DateTime:TDateTime;
 begin
  lStr:='';
  if Time>0 then
   begin
    DateTime:=FileDateToDateTime(Time);
    DateTimeToString(lStr,'dd.mm.yyyy hh:nn',DateTime);
   end;
  GetTimeStr:=lStr;
 end;

 function TFileRec.GetSortTime:string;
 var lStr:string;
     DateTime:TDateTime;
 begin
  lStr:='';
  if Time>0 then
   begin
    DateTime:=FileDateToDateTime(Time);
    DateTimeToString(lStr,'yyyymmddhhnn',DateTime);
   end;
  Result:=lStr;
 end;

function TFileRec.GetSortSize:string;
 var i:byte;
     sPom:string;
 begin
  sPom:=IntToStr(Size);
  i:=Length(sPom);
  while i<15 do
   begin
    sPom:=' '+sPom;
    inc(i);
   end;
  Result:=sPom;
 end;

 function TFileRec.GetSizeStr:string;
 begin
  if (Attr and faDirectory)>0 then
   GetSizeStr:='<DIR>'
  else
   GetSizeStr:=IntToStr(Size);
 end;

procedure TFileRec.SplitNameExt;

 function ExtractName(s:string):string;
  var lStr:string;
      i:byte;
  begin
   if s='..' then
    ExtractName:=s
   else
    begin
     lStr:='';
     i:=1;
     while (i<=Length(s)) and (s[i]<>'.') do
      begin
       lStr:=lStr+s[i];
       inc(i);
      end;
     ExtractName:=lStr;
    end;
  end;

 begin
  Ext:=Copy(ExtractFileExt(Name),2,3);
  Name:=ExtractName(Name);
 end;

function TFileRec.GetAtribStr:TAttrStr;
 var lStr:TAttrStr;
 begin
  if (Attr and faReadOnly)>0 then lStr:='r'
                             else lStr:='-';
  if (Attr and faArchive)>0 then lStr:=lStr+'a'
                            else lStr:=lStr+'-';
  if (Attr and faHidden)>0 then lStr:=lStr+'h'
                           else lStr:=lStr+'-';
  if (Attr and faSysFile)>0 then lStr:=lStr+'s'
                            else lStr:=lStr+'-';
  GetAtribStr:=lStr;
 end;

procedure TFileRec.SetValues(S :TSearchRec);
 begin
  Attr:=S.Attr;
  Time:=S.Time;
  Size:=S.Size;
  Name:=S.Name;
  Selected:=false;
  SplitNameExt;
 end;

function TFileRec.GetString(Sort:TSortType):string;
 var pomstr:string;
 begin
  if (Attr and faDirectory)>0 then pomstr:='1 '
                              else pomstr:='2 ';
  case Sort of
   stNone,stName: GetString:=pomstr+GetName;
   stExt        : GetString:=pomstr+Ext+'.'+Name;
   stSize       : GetString:=pomstr+GetSortSize+' '+GetName;
   stTime       : GetString:=pomstr+GetSortTime+' '+GetName;
  end;
 end;

function TFileRec.GetName:string;
 begin
  if Ext='' then GetName:=LowerCase(Name)
            else GetName:=LowerCase(Name+'.'+Ext);
 end;

end.
