unit PFDrives;

interface

uses SysUtils;

 type
  TDriveType = (dtUnknown, dtNoDrive, dtFloppy, dtFixed, dtNetwork, dtCDROM,
                dtRAM,dtPortfolio);

  TDriveRange = 'A'..'Z';
  TVolumeLabel = string[11];
  TDrive=record
   DriveType: TDriveType;
   PFDrive: char;
  end;

  TFileMode = (fmRead,fmWrite,fmReadWrite);
  TPFFile=record
   Handle: file;
   PF    : boolean;
  end;

 var
  Drives: array[TDriveRange] of TDrive;

 procedure DetectDrives;
 function GetVolumeLabel(Drive: Char): TVolumeLabel;
 procedure SetVolumeLabel(NewLabel: TVolumeLabel; Drive: Char);
 function FndFirst(const Path: string; Attr: Word; var SearchRec: TSearchRec): Integer;
 function FndNext(var SearchRec: TSearchRec): Integer;
 function FndFirstU(const Path: string; Attr: Word; var SearchRec: TSearchRec): Integer;
 function FndNextU(var SearchRec: TSearchRec): Integer;
 procedure FndClose(var SearchRec: TSearchRec);
 procedure MakeDirectory(Path:string);
 procedure SetSysDateTime(DateTime:TDateTime);
 procedure GetDiskSizes(Drive: char;var DS,DF: longint);
 function WildcardMatch(const filename, wildcard: string): Boolean;
 procedure FOpen(var f: TPFFile; FileName: string; Mode: TFileMode);
 procedure FClose(f: TPFFile);
 procedure BlckRead(f: TPFFile; var Buf; Count: word; var Result: word);
 procedure BlckWrite(f: TPFFile; var Buf; Count: word; var Result: word);
 procedure RmvDir(S: string);
 function RnmFile(OldName, NewName: string): Boolean;
 function DelFile(FileName: string): Boolean;
 function FileSetAttrDate(FileName: string; Attr: word; DateTime: longint): Boolean;

implementation

uses WinProcs, WinTypes, Pf;

var PortDrives: string[26];
    PortIndex: byte;
    FndDrive: char;


procedure SetSysDateTime(DateTime:TDateTime);
 begin
  PFForm.PFSetSysDateTime(DateTime);
 end;

procedure MakeDirectory(Path:string);
 var PFPath:string;
     PomDrive:char;
 begin
  PomDrive:=UpCase(Path[1]);
  PFPath:=Path;
  PFPath[1]:=Drives[PomDrive].PFDrive;
  if Drives[PomDrive].DriveType=dtPortfolio then
   PFForm.PFMkDir(PFPath)
  else
   MkDir(Path);
 end;

function FndFirst(const Path: string; Attr: Word; var SearchRec: TSearchRec): Integer;
 var PFPath:string;
 begin
  FndDrive:=UpCase(Path[1]);
  PFPath:=Path;
  PFPath[1]:=Drives[FndDrive].PFDrive;
  if Drives[FndDrive].DriveType=dtPortfolio then
   Result:=PFForm.PFFindFirstBuf(PFPath,Attr,SearchRec)
  else
   Result:=FindFirst(Path,Attr,SearchRec);
 end;

function FndNext(var SearchRec: TSearchRec): Integer;
 begin
  if Drives[FndDrive].DriveType=dtPortfolio then
   Result:=PFForm.PFFindNextBuf(SearchRec)
  else
   Result:=FindNext(SearchRec);
 end;

 function FndFirstU(const Path: string; Attr: Word; var SearchRec: TSearchRec): Integer;
 var PFPath:string;
 begin
  FndDrive:=UpCase(Path[1]);
  PFPath:=Path;
  PFPath[1]:=Drives[FndDrive].PFDrive;
  if Drives[FndDrive].DriveType=dtPortfolio then
   Result:=PFForm.PFFindFirst(PFPath,Attr,SearchRec)
  else
   Result:=FindFirst(Path,Attr,SearchRec);
 end;

function FndNextU(var SearchRec: TSearchRec): Integer;
 begin
  if Drives[FndDrive].DriveType=dtPortfolio then
   Result:=PFForm.PFFindNext(SearchRec)
  else
   Result:=FindNext(SearchRec);
 end;

procedure FndClose(var SearchRec: TSearchRec);
 begin
  if Drives[FndDrive].DriveType<>dtPortfolio then
   FindClose(SearchRec);
 end;

function DriveNumber(Drive: Char): Integer;
begin
  { A = 1, B = 2 ... }
  Result := Ord(UpCase(Drive)) - Ord('A') + 1;
end;

function WinDriveNumber(Drive: Char): Integer;
begin
  { A = 0, B = 1 ... }
  Result := DriveNumber(Drive) - 1;
end;

function IsCDROM(DriveNum: Integer): Boolean; assembler;
asm
        MOV     AX,1500h  { look for MSCDEX }
        XOR     BX,BX
        INT     2Fh
        OR      BX,BX
        JZ      @Finish
        MOV     AX,150Bh  { check for using CD driver }
        MOV     CX,DriveNum
        INT     2Fh
        OR      AX,AX
@Finish:
end;

function IsRAMDrive(DriveNum: Integer): Boolean; assembler;
var
  Temp: Boolean;
asm
        MOV     Temp,False
        PUSH    DS
        MOV     BX,SS
        MOV     DS,BX
        SUB     SP,0200h
        MOV     BX,SP
        MOV     AX,DriveNum
        MOV     CX,1
        XOR     DX,DX
        INT     25h  { read boot sector }
        ADD     SP,2
        JC      @@1
        MOV     BX,SP
        CMP     BYTE PTR SS:[BX+15h],0F8h  { reverify fixed disk }
        JNE     @@1
        CMP     BYTE PTR SS:[BX+10h],1  { check for single FAT }
        JNE     @@1
        MOV     Temp,True
@@1:    ADD     SP,0200h
        POP     DS
        MOV     AL,Temp
end;

function FindDriveType(aDrive: Char): TDrive;
 var
   n: Integer;
   D: TDrive;
 begin
   n := WinDriveNumber(aDrive);
   D.PFDrive:=#0;
   case GetDriveType(n) of
     0               : if (aDrive<'C') or (PortIndex>Length(PortDrives)) then
                        D.DriveType := dtNoDrive
                       else
                        begin
                         D.DriveType := dtPortFolio;
                         D.PFDrive:=PortDrives[PortIndex];
                         Inc(PortIndex);
                        end;
     DRIVE_REMOVABLE : D.DriveType := dtFloppy;
     DRIVE_FIXED     : if IsRAMDrive(n) then D.DriveType := dtRAM
                       else D.DriveType := dtFixed;
     DRIVE_REMOTE    : if IsCDROM(n) then D.DriveType := dtCDROM
                       else D.DriveType := dtNetwork;
   end;
   Result:=D;
 end;

 procedure DetectDrives;
 var
   d: Char;
 begin
  try
   PFForm.PFGetDrives(PortDrives);
  except
   PortDrives:='';
  end;
  PortIndex:=1;
  for d := 'A' to 'Z' do
   Drives[d] := FindDriveType(d);
 end;

function GetNetworkVolume(Drive: Char): string;
const
  LocalName: array[0..2] of Char = 'C:' + #0;
var
  BufSize: Word;
  Buf: array[0..128] of Char;
  s1: string[12];
  s2: string[79];
begin
  LocalName[0] := Drive;
{  BufSize := 127;
  if WNetGetConnection(LocalName, Buf, @BufSize) = WN_SUCCESS then
  begin   3.1 New format
    if Pos('/', StrPas(Buf)) > 0 then
      Unformat(StrPas(Buf), '%s/%s', [@s1, 15, @s2, 79])
    else
      Unformat(StrPas(Buf), '\\%s\%s', [@s1, 15, @s2, 79]);
    Result := Capitalize(s2) + ' on ''' + Capitalize(s1) + '''';
  end
  else
    Result := Capitalize(Trim(GetVolumeLabel(Drive)));}
end;

procedure PadVolumeLabel(var Volume: TVolumeLabel);
var
  i: Integer;
begin
  for i := Length(Volume) + 1 to 11 do Volume[i] := ' ';
  Volume[0] := Chr(11);
end;

function GetLocalVolume(Drive: Char): TVolumeLabel;
var
  rec: TSearchRec;
begin
  Result := '';
  if FindFirst(Drive + ':\*.*', faVolumeID, rec) = 0 then
  begin
    Result := rec.Name;
    if Pos('.', Result) > 0 then Delete(Result, 9, 1);
  end;
  FindClose(rec); { 3.11 }
end;

function GetPFVolume(Drive: Char): TVolumeLabel;
var
  rec: TSearchRec;
begin
  Result := '';
  if PFForm.PFFindFirst(Drives[UpCase(Drive)].PFDrive + ':\????????.???', faVolumeID, rec) = 0 then
   begin
    if Pos(' ',rec.Name)>0 then Result:=Copy(rec.Name,1,Pos(' ',rec.Name)-1)
                           else Result := rec.Name;
    if Pos('.', Result) > 0 then Delete(Result, 9, 1);
   end;
end;

function GetVolumeLabel(Drive: Char): TVolumeLabel;
 begin
  if (UpCase(Drive)<'A') or (UpCase(Drive)>'Z') then Exit;
  case Drives[UpCase(Drive)].DriveType of
  dtUnknown,
  dtNoDrive   : Result:='';
  dtFloppy,
  dtFixed,
  dtCDROM,
  dtRAM       : Result:=GetLocalVolume(Drive);
  dtNetwork   : Result:=GetNetworkVolume(Drive);
  dtPortfolio : Result:=GetPFVolume(Drive);
  end;
 end;

{ Procedures for getting, setting and deleting and deleting volume
  labels from a floppy or hard disk.  GetVolumeLabel uses FindFirst
  function.  SetVolumeLabel and DeleteVolumeLabel involves calling
  DOS interrupt 21h, functions 16h and 13h respectively. Since
  Windows does not support function 16h, it must be called using
  DPMI interrupt 31h, function 300h. }

type
  EDPMIError = class(Exception);
  EVolumeLabelError = class(Exception);

  PRealModeRegs = ^TRealModeRegs;
  TRealModeRegs = record
    case Integer of
      0: (EDI, ESI, EBP, EXX, EBX, EDX, ECX, EAX: Longint;
          Flags, ES, DS, FS, GS, IP, CS, SP, SS: Word);
      1: (DI, DIH, SI, SIH, BP, BPH, XX, XXH: Word;
          case Integer of
            0: (BX, BXH, DX, DXH, CX, CXH, AX, AXH: Word);
            1: (BL, BH, BLH, BHH, DL, DH, DLH, DHH,
                CL, CH, CLH, CHH, AL, AH, ALH, AHH: Byte));
  end;

  PExtendedFCB = ^TExtendedFCB;
  TExtendedFCB = Record
    ExtendedFCBflag: Byte;
    Reserved1      : array[1..5] of Byte;
    Attr           : Byte;
    DriveID        : Byte;
    FileName       : array[1..8] of Char;
    FileExt        : array[1..3] of Char;
    CurrentBlockNum: Word;
    RecordSize     : Word;
    FileSize       : LongInt;
    PackedDate     : Word;
    PackedTime     : Word;
    Reserved2      : array[1..8] of Byte;
    CurrentRecNum  : Byte;
    RandomRecNum   : LongInt;
  end;

{ RealModeInt invokes int 31h function 0300h to simulate
  a real mode interrupt from protected mode. }

procedure RealModeInt(Int: Byte; var Regs: TRealModeRegs);
var
  ErrorFlag: Boolean;
begin
  asm
    MOV     ErrorFlag,0      { assume success }
    MOV     AX,0300h         { function 300h }
    MOV     BL,Int           { real mode interrupt to execute }
    XOR     BH,BH            { (bh=0) required }
    XOR     CX,CX            { stack words to copy, assume zero }
    LES     DI,Regs          { es:di = Regs }
    INT     31h              { DPMI int 31h }
    JNC     @@End            { carry flag set on error }
  @@Error:
    MOV     ErrorFlag,1      { return false on error }
  @@End:
  end;
  if ErrorFlag then
    raise EDPMIError.Create('Failed to execute DPMI interrupt');
end;

procedure DeleteVolumeLabel(Drive: Char);
var
  CurName: TVolumeLabel;
  FCB: TExtendedFCB;
  ErrorFlag: WordBool;
begin
  { procedure deletes volume label from given drive }
  ErrorFlag := False;
  CurName := GetLocalVolume(Drive); { get current volume label }
  PadVolumeLabel(CurName);
  FillChar(FCB, SizeOf(FCB), 0);    { initialize FCB with zeros }
  with FCB do
  begin
    ExtendedFCBflag := $FF;         { always }
    Attr := faVolumeID;             { Volume ID attribute }
    DriveID := DriveNumber(Drive);  { Drive number }
    Move(CurName[1], FileName, 8);  { must enter volume label }
    Move(CurName[9], FileExt, 3);
  end;
  asm
    PUSH    DS                      { preserve ds }
    MOV     AX,SS                   { put seg of FCB (ss) in ds }
    MOV     DS,AX
    LEA     DX,FCB                  { put offset of FCB in dx }
    MOV     AX,1300h                { function 13h (delete file) }
    CALL    DOS3Call                { invoke int 21h }
    POP     DS                      { restore ds }
    CMP     AL,00h                  { check for success }
    JE      @@End
  @@Error:                          { set flag on error }
    MOV     ErrorFlag,1
  @@End:
  end;
  if ErrorFlag then
    raise EVolumeLabelError.Create('Unable to delete volume label');
end;

procedure SetLocalLabel(NewLabel: TVolumeLabel; Drive: Char);
var
  Regs: TRealModeRegs;
  FCB: PExtendedFCB;
  ErrorFlag: WordBool;
  Buf: Longint;
begin
  if NewLabel = '' then begin
    DeleteVolumeLabel(Drive);
    Exit;
  end;
  PadVolumeLabel(NewLabel);
  if GetVolumeLabel(Drive) <> '' then           { if has label... }
    DeleteVolumeLabel(Drive);                   { delete label }
  Buf := GlobalDOSAlloc(SizeOf(PExtendedFCB));  { allocate real buffer }
  try
    FCB := Ptr(LoWord(Buf), 0);
    FillChar(FCB^, SizeOf(FCB), 0);             { init FCB with zeros }
    with FCB^ do
    begin
      ExtendedFCBflag := $FF;                   { required }
      Attr := faVolumeID;                       { Volume ID attribute }
      DriveID := DriveNumber(Drive);            { Drive number }
      Move(NewLabel[1], FileName, 8);           { set new label }
      Move(NewLabel[9], FileExt, 3);
    end;
    FillChar(Regs, SizeOf(Regs), 0);
    with Regs do                                { SEGMENT of FCB }
    begin
      DS := HiWord(Buf);                        { offset = zero }
      DX := 0;
      AX := $1600;                              { function 16h }
    end;
    RealModeInt($21, Regs);                     { create file }
    if (Regs.AL <> 0) then                      { check for success }
      raise EVolumeLabelError.Create('Unable to create volume label');
    { 4.0 -- the created 'file' must be closed }
    FillChar(Regs, SizeOf(Regs), 0);
    with Regs do                                { SEGMENT of FCB }
    begin
      DS := HiWord(Buf);                        { offset = zero }
      DX := 0;
      AX := $1000;                              { function 10h }
    end;
    RealModeInt($21, Regs);                     { close file }
    if (Regs.AL <> 0) then                      { check for success }
      raise EVolumeLabelError.Create('Unable to close volume label');
  finally
    GlobalDOSFree(LoWord(Buf));
  end;
end;

 procedure SetVolumeLabel(NewLabel: TVolumeLabel; Drive: Char);
  begin
  if (UpCase(Drive)<'A') or (UpCase(Drive)>'Z') then Exit;
  case Drives[UpCase(Drive)].DriveType of
  dtUnknown,
  dtNoDrive   : ;
  dtFloppy,
  dtFixed,
  dtRAM       : SetLocalLabel(NewLabel, Drive);
  dtNetwork   : ;
  dtCDROM     : MessageBox(0,'Drive is read only.','Atari Portfolio Manager',mb_ok+mb_iconstop);
  dtPortfolio : PFForm.PFSetVolumeID(NewLabel, Drives[UpCase(Drive)].PFDrive);
  end;
  end;

 procedure GetDiskSizes(Drive: char;var DS,DF: longint);
 begin
  if Drives[Drive].DriveType=dtPortfolio then
   PFForm.PFGetDiskSizes(DriveNumber(Drives[Drive].PFDrive),DS,DF)
  else
   begin
    DS:=DiskSize(DriveNumber(Drive));
    DF:=DiskFree(DriveNumber(Drive));
   end;
 end;

function WildcardMatch(const filename, wildcard: string): Boolean;


 function WMatch(const filename, wildcard: string): Boolean;
 var
   i, j: Integer;
 begin
   Result := True;
   if wildcard = '*.*' then Exit; { 4.0 all files match }
   i := 1;
   j := 1;
   while (i <= Length(filename)) and (j <= Length(wildcard)) and Result do
   begin
     case wildcard[j] of
       '?' : begin
               Result := filename[i] <> '.';
               Inc(i);
               Inc(j);
             end;
       '*' : begin
               while (i <= Length(filename)) and (filename[i] <> '.') do Inc(i);
               Inc(j);
             end;
       else
       begin
         Result := filename[i] = wildcard[j];
         Inc(i);
         Inc(j);
       end;
     end;
   end;
   Result := Result and (i = Length(filename) + 1) and (j = Length(wildcard) + 1);
 end;

var lwc:string;
    li:byte;
    lres:boolean;
begin
 li:=0;
 lres:=false;
 while (li<Length(wildcard)) and not lres do
  begin
   inc(li);
   lwc:='';
   while (li<=Length(wildcard)) and (wildcard[li]<>';') do
    begin
     lwc:=lwc+wildcard[li];
     inc(li);
    end;
   lres:=WMatch(filename,lwc);
  end;
 Result:=lres;
end;

function RnmFile(OldName, NewName: string): Boolean;
 var DriveO,DriveN: char;
     PFOldName, PFNewName: string;
 begin
  if UpperCase(OldName)=UpperCase(NewName) then Exit;
  DriveO:=UpCase(OldName[1]);
  PFOldName:=OldName;
  PFOldName[1]:=Drives[DriveO].PFDrive;
  DriveN:=UpCase(NewName[1]);
  PFNewName:=NewName;
  PFNewName[1]:=Drives[DriveN].PFDrive;
  if (Drives[DriveO].DriveType=dtPortfolio) or (Drives[DriveN].DriveType=dtPortfolio) then
   Result:=(PFForm.PFRename(PFOldName,PFNewName)=0)
  else
   Result:=RenameFile(OldName,NewName);
 end;

function DelFile(FileName: string): Boolean;
 var Drive: char;
     PFPath: string;
 begin
  Drive:=UpCase(FileName[1]);
  PFPath:=FileName;
  PFPath[1]:=Drives[Drive].PFDrive;
  if Drives[Drive].DriveType=dtPortfolio then
   Result:=PFForm.PFDeleteFile(PFPath)=0
  else
   Result:=DeleteFile(FileName);
 end;

procedure RmvDir(S: string);
 var Drive: char;
     PFPath: string;
 begin
  Drive:=UpCase(S[1]);
  PFPath:=S;
  PFPath[1]:=Drives[Drive].PFDrive;
  if Drives[Drive].DriveType=dtPortfolio then
   PFForm.PFRmDir(PFPath)
  else
   RmDir(S);
 end;

procedure FOpen(var f: TPFFile; FileName: string; Mode: TFileMode);
 var Drive: char;
     PFPath: string;
 begin
  Drive:=UpCase(FileName[1]);
  PFPath:=FileName;
  PFPath[1]:=Drives[Drive].PFDrive;
  if Drives[Drive].DriveType=dtPortfolio then
   begin
    if PFForm.PFOpenFile(PFPath,Ord(Mode))<0 then
     raise EInOutError.Create('Cannot open file');
    f.PF:=true;
   end
  else
   begin
   AssignFile(f.Handle,FileName);
   f.PF:=false;
   FileMode:=Ord(Mode);
   if Mode=fmRead then Reset(f.Handle,1)
                  else Rewrite(f.Handle,1);
   end
 end;

procedure FClose(f: TPFFile);
 begin
  if f.PF then
   PFForm.PFCloseFile
  else
   CloseFile(f.Handle);
 end;

procedure BlckRead(f: TPFFile; var Buf; Count: word; var Result: word);
 begin
  if f.PF then
   PFForm.PFBlockRead(Buf,Count,Result)
  else
   BlockRead(f.Handle,Buf,Count,Result);
 end;

 procedure BlckWrite(f: TPFFile; var Buf; Count: word; var Result: word);
 begin
  if f.PF then
   PFForm.PFBlockWrite(Buf,Count,Result)
  else
   BlockWrite(f.Handle,Buf,Count,Result);
 end;

function FileSetAttrDate(FileName: string; Attr: word; DateTime: longint): Boolean;
 var f: TPFFile;
     Drive: char;
     FileHandle:integer;
 begin
  Drive:=UpCase(FileName[1]);
  if Drives[Drive].DriveType=dtPortfolio then
   begin
    FOpen(f,FileName,fmRead);
    Result:=PFForm.PFSetAttr(Attr,DateTime)=0;
    FClose(f);
   end
  else
   if FileSetAttr(FileName,Attr)=0 then
    begin
     if DateTime>0 then
      begin
       FileHandle:=FileOpen(FileName,fmOpenRead);
       FileSetDate(FileHandle, DateTime);
       FileClose(FileHandle);
      end;
     Result:=true;
    end
   else
    Result:=false;
 end;

end.
